@@ -20,15 +20,27 @@
 //     https://www.fltk.org/bugs.php
 //
 
-#include <FL/Fl.H>
-#include "Fl_Widget_Type.h"
+#include "Fl_Menu_Type.h"
+
+#include "fluid.h"
+#include "Fl_Window_Type.h"
 #include "alignment_panel.h"
+#include "file.h"
+#include "code.h"
+#include "Fluid_Image.h"
+#include "Shortcut_Button.h"
+
+#include <FL/Fl.H>
 #include <FL/fl_message.H>
 #include <FL/Fl_Menu_.H>
 #include <FL/Fl_Button.H>
 #include <FL/Fl_Value_Input.H>
 #include <FL/Fl_Text_Display.H>
+#include <FL/Fl_Menu_Button.H>
+#include <FL/Fl_Output.H>
+#include <FL/fl_draw.H>
 #include "../src/flstring.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -38,14 +50,6 @@ Fl_Menu_Item menu_item_type_menu[] = {
   {"Radio",0,0,(void*)FL_MENU_RADIO},
   {0}};
 
-extern int reading_file;
-extern int force_parent;
-extern int i18n_type;
-extern const char* i18n_include;
-extern const char* i18n_function;
-extern const char* i18n_file;
-extern const char* i18n_set;
-
 static char submenuflag;
 static uchar menuitemtype = 0;
 
@@ -96,8 +100,12 @@ void Fl_Input_Choice_Type::build_menu() {
   o->redraw();
 }
 
-
-Fl_Type *Fl_Menu_Item_Type::make() {
+/**
+ Create and add a new Menu Item node.
+ \param[in] strategy add after current or as last child
+ \return new Menu Item node
+ */
+Fl_Type *Fl_Menu_Item_Type::make(Strategy strategy) {
   // Find the current menu item:
   Fl_Type* q = Fl_Type::current;
   Fl_Type* p = q;
@@ -118,28 +126,43 @@ Fl_Type *Fl_Menu_Item_Type::make() {
   t->o = new Fl_Button(0,0,100,20);
   t->o->type(menuitemtype);
   t->factory = this;
-  t->add(p);
+  t->add(p, strategy);
   if (!reading_file) t->label(submenuflag ? "submenu" : "item");
   return t;
 }
 
-Fl_Type *Fl_Checkbox_Menu_Item_Type::make() {
+/**
+ Create and add a new Checkbox Menu Item node.
+ \param[in] strategy add after current or as last child
+ \return new node
+ */
+Fl_Type *Fl_Checkbox_Menu_Item_Type::make(Strategy strategy) {
     menuitemtype = FL_MENU_TOGGLE;
-    Fl_Type* t = Fl_Menu_Item_Type::make();
+    Fl_Type* t = Fl_Menu_Item_Type::make(strategy);
     menuitemtype = 0;
     return t;
 }
 
-Fl_Type *Fl_Radio_Menu_Item_Type::make() {
+/**
+ Create and add a new Radio ButtonMenu Item node.
+ \param[in] strategy add after current or as last child
+ \return new node
+ */
+Fl_Type *Fl_Radio_Menu_Item_Type::make(Strategy strategy) {
     menuitemtype = FL_MENU_RADIO;
-    Fl_Type* t = Fl_Menu_Item_Type::make();
+    Fl_Type* t = Fl_Menu_Item_Type::make(strategy);
     menuitemtype = 0;
     return t;
 }
 
-Fl_Type *Fl_Submenu_Type::make() {
+/**
+ Create and add a new Submenu Item node.
+ \param[in] strategy add after current or as last child
+ \return new node
+ */
+Fl_Type *Fl_Submenu_Type::make(Strategy strategy) {
   submenuflag = 1;
-  Fl_Type* t = Fl_Menu_Item_Type::make();
+  Fl_Type* t = Fl_Menu_Item_Type::make(strategy);
   submenuflag = 0;
   return t;
 }
@@ -174,8 +197,6 @@ const char* Fl_Menu_Item_Type::menu_name(int& i) {
   return unique_id(t, "menu", t->name(), t->label());
 }
 
-#include "Fluid_Image.h"
-
 void Fl_Menu_Item_Type::write_static() {
   if (callback() && is_name(callback()) && !user_defined(callback()))
     write_declare("extern void %s(Fl_Menu_*, %s);", callback(),
@@ -206,8 +227,9 @@ void Fl_Menu_Item_Type::write_static() {
     const char* ut = user_data_type() ? user_data_type() : "void*";
     write_c(", %s", ut);
     if (use_v) write_c(" v");
-    write_c(") {\n  %s", callback());
-    if (*(d-1) != ';') {
+    write_c(") {\n");
+    write_c_indented(callback());
+    if (*(d-1) != ';' && *(d-1) != '}') {
       const char *p = strrchr(callback(), '\n');
       if (p) p ++;
       else p = callback();
@@ -218,7 +240,7 @@ void Fl_Menu_Item_Type::write_static() {
     write_c("\n}\n");
     if (k) {
       write_c("void %s::%s(Fl_Menu_* o, %s v) {\n", k, cn, ut);
-      write_c("  ((%s*)(o", k);
+      write_c("%s((%s*)(o", indent(1), k);
       Fl_Type* t = parent; while (t->is_menu_item()) t = t->parent;
       Fl_Type *q = 0;
       // Go up one more level for Fl_Input_Choice, as these are groups themselves
@@ -345,8 +367,8 @@ void Fl_Menu_Item_Type::write_code1() {
   if (!prev->is_menu_item()) {
     // for first menu item, declare the array
     if (class_name(1)) {
-      if (i18n_type) write_h("  static unsigned char %s_i18n_done;\n", mname);
-      write_h("  static Fl_Menu_Item %s[];\n", mname);
+      if (i18n_type) write_h("%sstatic unsigned char %s_i18n_done;\n", indent(1), mname);
+      write_h("%sstatic Fl_Menu_Item %s[];\n", indent(1), mname);
     } else {
       if (i18n_type) write_h("extern unsigned char %s_i18n_done;\n", mname);
       write_h("extern Fl_Menu_Item %s[];\n", mname);
@@ -357,7 +379,7 @@ void Fl_Menu_Item_Type::write_code1() {
   if (c) {
     if (class_name(1)) {
       write_public(public_);
-      write_h("  static Fl_Menu_Item *%s;\n", c);
+      write_h("%sstatic Fl_Menu_Item *%s;\n", indent(1), c);
     } else {
       if (c==name())
         write_h("#define %s (%s+%d)\n", c, mname, i);
@@ -371,8 +393,8 @@ void Fl_Menu_Item_Type::write_code1() {
       const char* cn = callback_name();
       const char* ut = user_data_type() ? user_data_type() : "void*";
       write_public(0);
-      write_h("  inline void %s_i(Fl_Menu_*, %s);\n", cn, ut);
-      write_h("  static void %s(Fl_Menu_*, %s);\n", cn, ut);
+      write_h("%sinline void %s_i(Fl_Menu_*, %s);\n", indent(1), cn, ut);
+      write_h("%sstatic void %s(Fl_Menu_*, %s);\n", indent(1), cn, ut);
     }
   }
 
@@ -392,7 +414,7 @@ void Fl_Menu_Item_Type::write_code1() {
         menuItemInitialized = 1;
         write_c("%s{ Fl_Menu_Item* o = &%s[%d];\n", indent(), mname, i);
       }
-      write_c("%s  %s\n", indent(), extra_code(n));
+      write_c("%s%s\n", indent_plus(1), extra_code(n));
     }
   }
   if (menuItemInitialized)
@@ -489,21 +511,21 @@ void Fl_Menu_Type::write_code2() {
       }
       if (nLabel) {
         write_c("%sif (!%s_i18n_done) {\n", indent(), mName);
-        write_c("%s  int i=0;\n", indent());
-        write_c("%s  for ( ; i<%d; i++)\n", indent(), nItem);
-        write_c("%s    if (%s[i].label())\n", indent(), mName);
+        write_c("%sint i=0;\n", indent_plus(1));
+        write_c("%sfor ( ; i<%d; i++)\n", indent_plus(1), nItem);
+        write_c("%sif (%s[i].label())\n", indent_plus(2), mName);
         switch (i18n_type) {
           case 1:
-            write_c("%s      %s[i].label(%s(%s[i].label()));\n",
-                    indent(), mName, i18n_function, mName);
+            write_c("%s%s[i].label(%s(%s[i].label()));\n",
+                    indent_plus(3), mName, i18n_function, mName);
             break;
           case 2:
-            write_c("%s      %s[i].label(catgets(%s,%s,i+%d,%s[i].label()));\n",
-                    indent(), mName, i18n_file[0] ? i18n_file : "_catalog",
+            write_c("%s%s[i].label(catgets(%s,%s,i+%d,%s[i].label()));\n",
+                    indent_plus(3), mName, i18n_file[0] ? i18n_file : "_catalog",
                     i18n_set, mi->msgnum(), mName);
             break;
         }
-        write_c("%s  %s_i18n_done = 1;\n", indent(), mName);
+        write_c("%s%s_i18n_done = 1;\n", indent_plus(1), mName);
         write_c("%s}\n", indent());
       }
     }
@@ -525,7 +547,6 @@ void Fl_Menu_Type::copy_properties() {
 
 ////////////////////////////////////////////////////////////////
 
-#include <FL/Fl_Menu_Button.H>
 Fl_Menu_Item button_type_menu[] = {
   {"normal",0,0,(void*)0},
   {"popup1",0,0,(void*)Fl_Menu_Button::POPUP1},
@@ -582,51 +603,6 @@ Fl_Menu_Bar_Type Fl_Menu_Bar_type;
 ////////////////////////////////////////////////////////////////
 // Shortcut entry item in panel:
 
-#include <FL/Fl_Output.H>
-#include "Shortcut_Button.h"
-#include <FL/fl_draw.H>
-
-void Shortcut_Button::draw() {
-  if (value()) draw_box(FL_DOWN_BOX, (Fl_Color)9);
-  else draw_box(FL_UP_BOX, FL_WHITE);
-  fl_font(FL_HELVETICA,14); fl_color(FL_FOREGROUND_COLOR);
-        if (use_FL_COMMAND && (svalue & (FL_CTRL|FL_META))) {
-                char buf[1024];
-                fl_snprintf(buf, 1023, "Command+%s", fl_shortcut_label(svalue&~(FL_CTRL|FL_META)));
-                fl_draw(buf,x()+6,y(),w(),h(),FL_ALIGN_LEFT);
-        } else {
-                fl_draw(fl_shortcut_label(svalue),x()+6,y(),w(),h(),FL_ALIGN_LEFT);
-        }
-}
-
-int Shortcut_Button::handle(int e) {
-  when(0); type(FL_TOGGLE_BUTTON);
-  if (e == FL_KEYBOARD) {
-    if (!value()) return 0;
-    int v = Fl::event_text()[0];
-    if ( (v > 32 && v < 0x7f) || (v > 0xa0 && v <= 0xff) ) {
-      if (isupper(v)) {
-        v = tolower(v);
-        v |= FL_SHIFT;
-      }
-      v = v | (Fl::event_state()&(FL_META|FL_ALT|FL_CTRL));
-    } else {
-      v = (Fl::event_state()&(FL_META|FL_ALT|FL_CTRL|FL_SHIFT)) | Fl::event_key();
-      if (v == FL_BackSpace && svalue) v = 0;
-    }
-    if (v != svalue) {svalue = v; set_changed(); redraw(); do_callback(); }
-    return 1;
-  } else if (e == FL_UNFOCUS) {
-    int c = changed(); value(0); if (c) set_changed();
-    return 1;
-  } else if (e == FL_FOCUS) {
-    return value();
-  } else {
-    int r = Fl_Button::handle(e);
-    if (e == FL_RELEASE && value() && Fl::focus() != this) take_focus();
-    return r;
-  }
-}
 
 void shortcut_in_cb(Shortcut_Button* i, void* v) {
   if (v == LOAD) {
