@@ -14,17 +14,30 @@
 //     https://www.fltk.org/bugs.php
 //
 
+#include "Fl_Widget_Type.h"
+
+#include "fluid.h"
+#include "Fl_Window_Type.h"
+#include "Fl_Group_Type.h"
+#include "Fl_Menu_Type.h"
+#include "Fl_Function_Type.h"
+#include "file.h"
+#include "code.h"
+#include "Fluid_Image.h"
+#include "alignment_panel.h"
+#include "widget_panel.h"
+
 #include <FL/Fl.H>
 #include <FL/Fl_Group.H>
 #include <FL/Fl_Table.H>
 #include <FL/Fl_Input.H>
-#include "Fl_Widget_Type.h"
-#include "alignment_panel.h"
 #include <FL/fl_message.H>
 #include <FL/Fl_Slider.H>
 #include <FL/Fl_Spinner.H>
 #include <FL/Fl_Window.H>
+#include <FL/fl_show_colormap.H>
 #include "../src/flstring.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -35,16 +48,6 @@
 // instance, sets the widget pointers, and makes all the display
 // update correctly...
 
-extern int reading_file;
-int force_parent;
-extern int gridx;
-extern int gridy;
-extern int i18n_type;
-extern const char* i18n_include;
-extern const char* i18n_function;
-extern const char* i18n_file;
-extern const char* i18n_set;
-
 int Fl_Widget_Type::default_size = FL_NORMAL_SIZE;
 
 int Fl_Widget_Type::is_widget() const {return 1;}
@@ -89,7 +92,12 @@ Fl_Widget_Type::ideal_spacing(int &x, int &y) {
     x = y = 10;
 }
 
-Fl_Type *Fl_Widget_Type::make() {
+/**
+ Make a new Widget node.
+ \param[in] strategy is kAddAsLastChild or kAddAfterCurrent
+ \return new node
+ */
+Fl_Type *Fl_Widget_Type::make(Strategy strategy) {
   // Find the current widget, or widget to copy:
   Fl_Type *qq = Fl_Type::current;
   while (qq && (!qq->is_widget() || qq->is_menu_item())) qq = qq->parent;
@@ -159,13 +167,11 @@ Fl_Type *Fl_Widget_Type::make() {
   // Put it in the parent:
   //  ((Fl_Group *)(p->o))->add(t->o); (done by Fl_Type::add())
   // add to browser:
-  t->add(p);
+  t->add(p, strategy);
   t->redraw();
   return t;
 }
 
-#include "Fluid_Image.h"
-
 void Fl_Widget_Type::setimage(Fluid_Image *i) {
   if (i == image || is_window()) return;
   if (image) image->decrement();
@@ -287,9 +293,6 @@ Fl_Type *sort(Fl_Type *parent) {
 ////////////////////////////////////////////////////////////////
 // The control panels!
 
-#include "widget_panel.h"
-#include <FL/fl_show_colormap.H>
-
 static Fl_Window *the_panel;
 
 // All the callbacks use the argument to indicate whether to load or store.
@@ -1728,21 +1731,30 @@ void value_cb(Fl_Value_Input* i, void* v) {
 Fl_Menu_Item *Fl_Widget_Type::subtypes() {return 0;}
 
 void subtype_cb(Fl_Choice* i, void* v) {
+  static Fl_Menu_Item empty_type_menu[] = {
+    {"Normal",0,0,(void*)0},
+    {0}};
+
   if (v == LOAD) {
     Fl_Menu_Item* m = current_widget->subtypes();
-    if (!m) {i->deactivate(); return;}
-    i->menu(m);
-    int j;
-    for (j = 0;; j++) {
-      if (!m[j].text) {j = 0; break;}
-      if (current_widget->is_spinner()) {
-        if (m[j].argument() == ((Fl_Spinner*)current_widget->o)->type()) break;
-      } else {
-        if (m[j].argument() == current_widget->o->type()) break;
+    if (!m) {
+      i->menu(empty_type_menu);
+      i->value(0);
+      i->deactivate();
+    } else {
+      i->menu(m);
+      int j;
+      for (j = 0;; j++) {
+        if (!m[j].text) {j = 0; break;}
+        if (current_widget->is_spinner()) {
+          if (m[j].argument() == ((Fl_Spinner*)current_widget->o)->type()) break;
+        } else {
+          if (m[j].argument() == current_widget->o->type()) break;
+        }
       }
+      i->value(j);
+      i->activate();
     }
-    i->value(j);
-    i->activate();
     i->redraw();
   } else {
     int mod = 0;
@@ -1834,7 +1846,7 @@ void live_mode_cb(Fl_Button*o,void *) {
         Fl_Group::current(0);
         int w = live_widget->w();
         int h = live_widget->h();
-        live_window = new Fl_Double_Window(w+20, h+55, "Fluid Live Mode Widget");
+        live_window = new Fl_Double_Window(w+20, h+55, "Fluid Live Resize");
         live_window->box(FL_FLAT_BOX);
         live_window->color(FL_GREEN);
         Fl_Group *rsz = new Fl_Group(0, h+20, 130, 35);
@@ -1842,7 +1854,7 @@ void live_mode_cb(Fl_Button*o,void *) {
         Fl_Box *rsz_dummy = new Fl_Box(110, h+20, 1, 25);
         rsz_dummy->box(FL_NO_BOX);
         rsz->resizable(rsz_dummy);
-        Fl_Button *btn = new Fl_Button(10, h+20, 100, 25, "Exit Live Mode");
+        Fl_Button *btn = new Fl_Button(10, h+20, 100, 25, "Exit Live Resize");
         btn->labelsize(12);
         btn->callback(leave_live_mode_cb);
         rsz->end();
@@ -1861,6 +1873,7 @@ void live_mode_cb(Fl_Button*o,void *) {
             live_window->size_range(mw, mh, MW, MH);
         }
         live_window->show();
+        live_widget->show();
       } else o->value(0);
     } else o->value(0);
   } else {
@@ -1906,8 +1919,6 @@ void Fl_Widget_Type::open() {
   if (numselected) the_panel->show();
 }
 
-Fl_Type *Fl_Type::current;
-
 extern void redraw_overlays();
 extern void check_redraw_corresponding_parent(Fl_Type*);
 extern void redraw_browser();
@@ -1954,7 +1965,8 @@ void selection_changed(Fl_Type *p) {
 
 // test to see if user named a function, or typed in code:
 int is_name(const char *c) {
-  for (; *c; c++) if (ispunct(*c) && *c!='_' && *c!=':') return 0;
+  for (; *c; c++)
+    if ((ispunct(*c)||*c=='\n') && *c!='_' && *c!=':') return 0;
   return 1;
 }
 
@@ -2054,8 +2066,9 @@ void Fl_Widget_Type::write_static() {
     const char* ut = user_data_type() ? user_data_type() : "void*";
     write_c(", %s", ut);
     if (use_v) write_c(" v");
-    write_c(") {\n  %s", callback());
-    if (*(d-1) != ';') {
+    write_c(") {\n");
+    write_c_indented(callback());
+    if (*(d-1) != ';' && *(d-1) != '}') {
       const char *p = strrchr(callback(), '\n');
       if (p) p ++;
       else p = callback();
@@ -2066,7 +2079,7 @@ void Fl_Widget_Type::write_static() {
     write_c("\n}\n");
     if (k) {
       write_c("void %s::%s(%s* o, %s v) {\n", k, cn, t, ut);
-      write_c("  ((%s*)(o", k);
+      write_c("%s((%s*)(o", indent(1), k);
       Fl_Type *q = 0;
       for (Fl_Type* p = parent; p && p->is_widget(); q = p, p = p->parent)
         write_c("->parent()");
@@ -2089,11 +2102,6 @@ void Fl_Widget_Type::write_static() {
   }
 }
 
-const char *Fl_Type::callback_name() {
-  if (is_name(callback())) return callback();
-  return unique_id(this, "cb", name(), label());
-}
-
 extern int varused_test, varused;
 
 void Fl_Widget_Type::write_code1() {
@@ -2102,15 +2110,15 @@ void Fl_Widget_Type::write_code1() {
   if (c) {
     if (class_name(1)) {
       write_public(public_);
-      write_h("  %s *%s;\n", t, c);
+      write_h("%s%s *%s;\n", indent(1), t, c);
     }
   }
   if (class_name(1) && callback() && !is_name(callback())) {
     const char* cn = callback_name();
     const char* ut = user_data_type() ? user_data_type() : "void*";
     write_public(0);
-    write_h("  inline void %s_i(%s*, %s);\n", cn, t, ut);
-    write_h("  static void %s(%s*, %s);\n", cn, t, ut);
+    write_h("%sinline void %s_i(%s*, %s);\n", indent(1), cn, t, ut);
+    write_h("%sstatic void %s(%s*, %s);\n", indent(1), cn, t, ut);
   }
   // figure out if local variable will be used (prevent compiler warnings):
   int wused = !name() && is_window();
@@ -2189,7 +2197,7 @@ void Fl_Widget_Type::write_code1() {
   }
   write_c(");\n");
 
-  indentation += 2;
+  indentation++;
 
   if (wused) write_c("%sw = o; if (w) {/* empty */}\n", indent());
 
@@ -2369,9 +2377,6 @@ void Fl_Widget_Type::write_widget_code() {
     if (i & FL_ALIGN_INSIDE) write_c("|FL_ALIGN_INSIDE");
     write_c("));\n");
   }
-  // avoid the unsupported combination of flegs when user sets
-  // "when" to "FL_WHEN_NEVER", but keeps the "no change" set.
-  // FIXME: This could be reflected in the GUI by graying out the button.
   Fl_When ww = o->when();
   if (ww==FL_WHEN_NOT_CHANGED)
     ww = FL_WHEN_NEVER;
@@ -2401,7 +2406,7 @@ void Fl_Widget_Type::write_extra_code() {
 }
 
 void Fl_Widget_Type::write_block_close() {
-  indentation -= 2;
+  indentation--;
   write_c("%s} // %s* %s\n", indent(), subclassname(this),
           name() ? name() : "o");
 }
@@ -2438,7 +2443,7 @@ void Fl_Widget_Type::write_properties() {
   if (is_spinner() && ((Fl_Spinner*)o)->type() != ((Fl_Spinner*)tplate)->type()) {
     write_string("type");
     write_word(item_name(subtypes(), ((Fl_Spinner*)o)->type()));
-  } else if (o->type() != tplate->type() || is_window()) {
+  } else if (subtypes() && (o->type() != tplate->type() || is_window())) {
     write_string("type");
     write_word(item_name(subtypes(), o->type()));
   }
@@ -2532,8 +2537,6 @@ void Fl_Widget_Type::write_properties() {
   }
 }
 
-int pasteoffset;
-extern double read_version;
 void Fl_Widget_Type::read_property(const char *c) {
   int x,y,w,h; Fl_Font f; int s; Fl_Color cc;
   if (!strcmp(c,"private")) {
@@ -2698,7 +2701,6 @@ Fl_Menu_Item boxmenu1[] = {
   {"11",                0,0,(void *)FL_DOWN_FRAME},
 {0}};
 
-extern int fdesign_flip;
 int lookup_symbol(const char *, int &, int numberok = 0);
 
 int Fl_Widget_Type::read_fdesign(const char* propname, const char* value) {
