@@ -18,6 +18,17 @@
 //     https://www.fltk.org/bugs.php
 //
 
+#include "Fl_Window_Type.h"
+
+#include "fluid.h"
+#include "widget_browser.h"
+#include "undo.h"
+#include "alignment_panel.h"
+#include "file.h"
+#include "code.h"
+#include "widget_panel.h"
+#include "factory.h"
+
 #include <FL/Fl.H>
 #include <FL/Fl_Overlay_Window.H>
 #include <FL/fl_message.H>
@@ -25,18 +36,12 @@
 #include <FL/platform.H>
 #include <FL/Fl_Menu_Item.H>
 #include <FL/Fl_Round_Button.H>
-#include "Fl_Widget_Type.h"
-#include "undo.h"
+#include "../src/flstring.h"
+
 #include <math.h>
 #include <stdlib.h>
-#include "alignment_panel.h"
 #include <stdio.h>
 
-extern int gridx;
-extern int gridy;
-extern int snap;
-extern int show_guides;
-
 int include_H_from_C = 1;
 int use_FL_COMMAND = 0;
 extern int i18n_type;
@@ -49,8 +54,6 @@ extern Fl_Preferences   fluid_prefs;
 
 inline int fl_min(int a, int b) { return (a < b ? a : b); }
 
-#include "widget_panel.h"
-
 // Update the XYWH values in the widget panel...
 static void update_xywh() {
   if (current_widget && current_widget->is_widget()) {
@@ -174,9 +177,6 @@ void i18n_int_cb(Fl_Int_Input *i, void *) {
   set_modflag(1);
 }
 
-extern const char* header_file_name;
-extern const char* code_file_name;
-
 void show_project_cb(Fl_Widget *, void *) {
   if(project_window==0) make_project_window();
   include_H_from_C_button->value(include_H_from_C);
@@ -350,7 +350,12 @@ int Overlay_Window::handle(int e) {
   return ret;
 }
 
-Fl_Type *Fl_Window_Type::make() {
+/**
+ Make and add a new WIndow node.
+ \param[in] strategy is kAddAsLastChild or kAddAfterCurrent
+ \return new node
+ */
+Fl_Type *Fl_Window_Type::make(Strategy strategy) {
   Fl_Type *p = Fl_Type::current;
   while (p && !p->is_code_block()) p = p->parent;
   if (!p) {
@@ -373,7 +378,7 @@ Fl_Type *Fl_Window_Type::make() {
   Overlay_Window *w = new Overlay_Window(100,100);
   w->window = myo;
   myo->o = w;
-  myo->add(p);
+  myo->add(p, strategy);
   myo->modal = 0;
   myo->non_modal = 0;
   return myo;
@@ -1090,7 +1095,7 @@ extern Fl_Menu_Item Main_Menu[];
 
 // Calculate new bounding box of selected widgets:
 void Fl_Window_Type::fix_overlay() {
-  Main_Menu[40].label("Hide O&verlays");
+  overlay_item->label("Hide O&verlays");
   overlays_invisible = 0;
   recalc = 1;
   ((Overlay_Window *)(this->o))->redraw_overlay();
@@ -1121,8 +1126,10 @@ void redraw_overlays() {
 void toggle_overlays(Fl_Widget *,void *) {
   overlays_invisible = !overlays_invisible;
 
-  if (overlays_invisible) Main_Menu[40].label("Show O&verlays");
-  else Main_Menu[40].label("Hide O&verlays");
+  if (overlays_invisible)
+    overlay_item->label("Show O&verlays");
+  else
+    overlay_item->label("Hide O&verlays");
 
   for (Fl_Type *o=Fl_Type::first; o; o=o->next)
     if (o->is_window()) {
@@ -1176,9 +1183,64 @@ void Fl_Window_Type::moveallchildren()
   update_xywh();
 }
 
+int Fl_Window_Type::popupx = 0x7FFFFFFF; // mark as invalid (MAXINT)
+int Fl_Window_Type::popupy = 0x7FFFFFFF;
+
 int Fl_Window_Type::handle(int event) {
-  static Fl_Type* selection;
+  static Fl_Type* selection = NULL;
   switch (event) {
+  case FL_DND_ENTER:
+    Fl::belowmouse(o);
+  case FL_DND_DRAG:
+    {
+      // find the innermost item clicked on:
+      selection = this;
+      for (Fl_Type* i=next; i && i->level>level; i=i->next)
+        if (i->is_group()) {
+          Fl_Widget_Type* myo = (Fl_Widget_Type*)i;
+          for (Fl_Widget *o1 = myo->o; o1; o1 = o1->parent())
+            if (!o1->visible()) goto CONTINUE_DND;
+          if (Fl::event_inside(myo->o)) {
+            selection = myo;
+            if (Fl::event_clicks()==1)
+              reveal_in_browser(myo);
+          }
+        }
+    CONTINUE_DND:
+      if (selection && !selection->selected) {
+        select_only(selection);
+        ((Overlay_Window *)o)->redraw_overlay();
+      }
+    }
+  case FL_DND_RELEASE:
+    return 1;
+  case FL_PASTE:
+    { Fl_Type *prototype = typename_to_prototype(Fl::event_text());
+      if (prototype==NULL)
+        return 0;
+
+      in_this_only = this;
+      popupx = Fl::event_x();
+      popupy = Fl::event_y();
+      // If the selected widget at dnd start and the drop target are the same,
+      // or in the same group, add after selection. Otherwise, just add
+      // at the end of the selected group.
+      if (   Fl_Type::current_dnd->group()
+          && selection->group()
+          && Fl_Type::current_dnd->group()==selection->group())
+      {
+        Fl_Type *cc = Fl_Type::current;
+        Fl_Type::current = Fl_Type::current_dnd;
+        add_new_widget_from_user(prototype, kAddAfterCurrent);
+        Fl_Type::current = cc;
+      } else {
+        add_new_widget_from_user(prototype, kAddAsLastChild);
+      }
+      popupx = 0x7FFFFFFF;
+      popupy = 0x7FFFFFFF; // mark as invalid (MAXINT)
+      in_this_only = NULL;
+      return 1;
+    }
   case FL_PUSH:
     x1 = mx = Fl::event_x();
     y1 = my = Fl::event_y();
@@ -1187,8 +1249,10 @@ int Fl_Window_Type::handle(int event) {
     if (Fl::event_button() >= 3) {
       in_this_only = this; // modifies how some menu items work.
       static const Fl_Menu_Item* myprev;
+      popupx = mx; popupy = my;
       const Fl_Menu_Item* m = New_Menu->popup(mx,my,"New",myprev);
       if (m && m->callback()) {myprev = m; m->do_callback(this->o);}
+      popupx = 0x7FFFFFFF; popupy = 0x7FFFFFFF; // mark as invalid (MAXINT)
       in_this_only = 0;
       return 1;
     }
@@ -1316,9 +1380,8 @@ int Fl_Window_Type::handle(int event) {
     case FL_Up:    dx = 0; dy = -1; goto ARROW;
     case FL_Down:  dx = 0; dy = +1; goto ARROW;
     ARROW:
-      // for some reason BOTTOM/TOP are swapped... should be fixed...
-      drag = (Fl::event_state(FL_SHIFT)) ? (RIGHT|TOP) : DRAG;
-      if (Fl::event_state(FL_CTRL)) {dx *= gridx; dy *= gridy;}
+      drag = (Fl::event_state(FL_SHIFT)) ? (RIGHT|BOTTOM) : DRAG;
+      if (Fl::event_state(FL_COMMAND)) {dx *= gridx; dy *= gridy;}
       moveallchildren();
       drag = 0;
       return 1;
@@ -1345,9 +1408,6 @@ int Fl_Window_Type::handle(int event) {
 
 ////////////////////////////////////////////////////////////////
 
-#include <stdio.h>
-#include "../src/flstring.h"
-
 void Fl_Window_Type::write_code1() {
   Fl_Widget_Type::write_code1();
 }
@@ -1388,7 +1448,6 @@ void Fl_Window_Type::write_properties() {
   if (o->visible()) write_string("visible");
 }
 
-extern int pasteoffset;
 void Fl_Window_Type::read_property(const char *c) {
   if (!strcmp(c,"modal")) {
     modal = 1;
@@ -1438,7 +1497,12 @@ int Fl_Window_Type::read_fdesign(const char* propname, const char* value) {
 Fl_Widget_Class_Type Fl_Widget_Class_type;
 Fl_Widget_Class_Type *current_widget_class = 0;
 
-Fl_Type *Fl_Widget_Class_Type::make() {
+/**
+ Create and add a new Widget Class node.
+ \param[in] strategy add after current or as last child
+ \return new node
+ */
+Fl_Type *Fl_Widget_Class_Type::make(Strategy strategy) {
   Fl_Type *p = Fl_Type::current;
   while (p && (!p->is_decl_block() || (p->is_widget() && p->is_class()))) p = p->parent;
   Fl_Widget_Class_Type *myo = new Fl_Widget_Class_Type();
@@ -1459,7 +1523,7 @@ Fl_Type *Fl_Widget_Class_Type::make() {
   Overlay_Window *w = new Overlay_Window(100,100);
   w->window = myo;
   myo->o = w;
-  myo->add(p);
+  myo->add(p, strategy);
   myo->modal = 0;
   myo->non_modal = 0;
   myo->wc_relative = 0;
@@ -1504,54 +1568,56 @@ void Fl_Widget_Class_Type::write_code1() {
   const char *c = subclass();
   if (!c) c = "Fl_Group";
 
+  write_c("\n");
   write_comment_h();
   write_h("\nclass %s : public %s {\n", name(), c);
   if (strstr(c, "Window")) {
-    write_h("  void _%s();\n", trimclassname(name()));
+    write_h("%svoid _%s();\n", indent(1), trimclassname(name()));
     write_h("public:\n");
-    write_h("  %s(int X, int Y, int W, int H, const char *L = 0);\n", trimclassname(name()));
-    write_h("  %s(int W, int H, const char *L = 0);\n", trimclassname(name()));
-    write_h("  %s();\n", trimclassname(name()));
+    write_h("%s%s(int X, int Y, int W, int H, const char *L = 0);\n", indent(1), trimclassname(name()));
+    write_h("%s%s(int W, int H, const char *L = 0);\n", indent(1), trimclassname(name()));
+    write_h("%s%s();\n", indent(1), trimclassname(name()));
 
     // a constructor with all four dimensions plus label
-    write_c("%s::%s(int X, int Y, int W, int H, const char *L)\n", name(), trimclassname(name()));
-    write_c("  : %s(X, Y, W, H, L) {\n", c);
-    write_c("  _%s();\n", trimclassname(name()));
+    write_c("%s::%s(int X, int Y, int W, int H, const char *L) :\n", name(), trimclassname(name()));
+    write_c("%s%s(X, Y, W, H, L)\n{\n", indent(1), c);
+    write_c("%s_%s();\n", indent(1), trimclassname(name()));
     write_c("}\n\n");
 
     // a constructor with just the size and label. The window manager will position the window
-    write_c("%s::%s(int W, int H, const char *L)\n", name(), trimclassname(name()));
-    write_c("  : %s(0, 0, W, H, L) {\n", c);
-    write_c("  clear_flag(16);\n");
-    write_c("  _%s();\n", trimclassname(name()));
+    write_c("%s::%s(int W, int H, const char *L) :\n", name(), trimclassname(name()));
+    write_c("%s%s(0, 0, W, H, L)\n{\n", indent(1), c);
+    write_c("%sclear_flag(16);\n", indent(1));
+    write_c("%s_%s();\n", indent(1), trimclassname(name()));
     write_c("}\n\n");
 
     // a constructor that takes size and label from the Fluid database
-    write_c("%s::%s()\n", name(), trimclassname(name()));
-    write_c("  : %s(0, 0, %d, %d, ", c, o->w(), o->h());
+    write_c("%s::%s() :\n", name(), trimclassname(name()));
+    write_c("%s%s(0, 0, %d, %d, ", indent(1), c, o->w(), o->h());
     const char *cstr = label();
     if (cstr) write_cstring(cstr);
     else write_c("0");
-    write_c(") {\n");
-    write_c("  clear_flag(16);\n");
-    write_c("  _%s();\n", trimclassname(name()));
+    write_c(")\n{\n");
+    write_c("%sclear_flag(16);\n", indent(1));
+    write_c("%s_%s();\n", indent(1), trimclassname(name()));
     write_c("}\n\n");
 
     write_c("void %s::_%s() {\n", name(), trimclassname(name()));
-//    write_c("  %s *w = this;\n", name());
+//    write_c("%s%s *w = this;\n", indent(1), name());
   } else {
     write_h("public:\n");
-    write_h("  %s(int X, int Y, int W, int H, const char *L = 0);\n", trimclassname(name()));
-
-    write_c("%s::%s(int X, int Y, int W, int H, const char *L)\n", name(), trimclassname(name()));
+    write_h("%s%s(int X, int Y, int W, int H, const char *L = 0);\n",
+            indent(1), trimclassname(name()));
+    write_c("%s::%s(int X, int Y, int W, int H, const char *L) :\n", name(), trimclassname(name()));
     if (wc_relative)
-      write_c("  : %s(0, 0, W, H, L) {\n", c);
+      write_c("%s%s(0, 0, W, H, L)\n{\n", indent(1), c);
     else
-      write_c("  : %s(X, Y, W, H, L) {\n", c);
+      write_c("%s%s(X, Y, W, H, L)\n{\n", indent(1), c);
   }
 
-//  write_c("  %s *o = this;\n", name());
+//  write_c("%s%s *o = this;\n", indent(1), name());
 
+  indentation++;
   write_widget_code();
 }
 
@@ -1569,6 +1635,7 @@ void Fl_Widget_Class_Type::write_code2() {
   write_c("%send();\n", indent());
   if (((Fl_Window*)o)->resizable() == o)
     write_c("%sresizable(this);\n", indent());
+  indentation--;
   write_c("}\n");
 }
 
